import 'package:flutter/material.dart';

/// Navigator相关
extension NavigatorExt on BuildContext {
  Future<T> push<T extends Object>(Route<T> route) => Navigator.of(this).push(route);

  Future<T> pushNamed<T extends Object>(String routeName, {Object arguments}) =>
      Navigator.of(this).pushNamed(routeName, arguments: arguments);

  Future<T> pushReplacement<T extends Object, TO extends Object>(Route<T> newRoute, {TO result}) =>
      Navigator.of(this).pushReplacement(newRoute, result: result);

  Future<T> pushReplacementNamed<T extends Object, TO extends Object>(String routeName, {TO result, Object arguments}) =>
      Navigator.of(this).pushReplacementNamed(routeName, result: result, arguments: arguments);

  Future<T> pushAndRemoveUntil<T extends Object>(Route<T> newRoute, RoutePredicate predicate) =>
      Navigator.of(this).pushAndRemoveUntil(newRoute, predicate);

  Future<T> pushNamedAndRemoveUntil<T extends Object>(String newRouteName, RoutePredicate predicate, {Object arguments}) =>
      Navigator.of(this).pushNamedAndRemoveUntil(newRouteName, predicate, arguments: arguments);

  void pop<T extends Object>([T result]) => Navigator.of(this).pop(result);

  void popUntil(RoutePredicate predicate) => Navigator.of(this).popUntil(predicate);

  Future<T> popAndPushNamed<T extends Object, TO extends Object>(String routeName, {TO result, Object arguments}) =>
      Navigator.of(this).popAndPushNamed(routeName, result: result, arguments: arguments);

  void removeRoute(Route<dynamic> route) => Navigator.of(this).removeRoute(route);

  void removeRouteBelow(Route<dynamic> anchorRoute) => Navigator.of(this).removeRouteBelow(anchorRoute);

  void replace<T extends Object>({@required Route<dynamic> oldRoute, @required Route<T> newRoute}) =>
      Navigator.of(this).replace(oldRoute: oldRoute, newRoute: newRoute);

  void replaceRouteBelow<T extends Object>({@required Route<dynamic> anchorRoute, @required Route<T> newRoute}) =>
      Navigator.of(this).replaceRouteBelow(anchorRoute: anchorRoute, newRoute: newRoute);

  bool canPop() => Navigator.of(this).canPop();
}

/// Desc: FocusScope相关
extension FocusScopeExt on BuildContext {
  void unfocus({UnfocusDisposition disposition = UnfocusDisposition.scope}) => FocusScope.of(this).unfocus(disposition: disposition);

  void requestFocus([FocusNode focusNode]) => FocusScope.of(this).requestFocus(focusNode);
}
