import 'dart:convert';
import 'dart:typed_data';
import 'package:convert/convert.dart';
import 'package:crypto/crypto.dart';
import 'package:yuro/util/datetime.dart';

/// String的扩展函数
extension StringExt on String {
  int parseToInt() => int.tryParse(this);

  bool isDigit() => int.tryParse(this) != null;

  double parseToDouble() => double.parse(this);

  // part of DateTime
  ///
  DateTime parseToDateTime() => DateUtil.parseStr(this);

  /// 判断日期是否是今天
  bool isToday([String format = DateUtil.FORMAT_DEFAULT]) => this == DateUtil.formatNow(format);

  // part of convert
  String toMd5([String salt = '']) => hex.encode(md5.convert(utf8.encode('${this}$salt')).bytes);

  String toSha1() => hex.encode(sha1.convert(utf8.encode(this)).bytes);

  String toSha256() => hex.encode(sha256.convert(utf8.encode(this)).bytes);

  String toSha512() => hex.encode(sha512.convert(utf8.encode(this)).bytes);

  String toBase64() => base64Encode(utf8.encode(this));

  String fromBase64() => utf8.decode(base64Decode(this));

  Uint8List toUint8List() => hex.decode(this);
}


