/// 日期工具类
class DateUtil {
  DateUtil._();

  static const int SECOND_IN_MILLIS = 1000;
  static const int MINUTE_IN_MILLIS = SECOND_IN_MILLIS * 60;
  static const int HOUR_IN_MILLIS = MINUTE_IN_MILLIS * 60;
  static const int DAY_IN_MILLIS = HOUR_IN_MILLIS * 24;
  static const int WEEK_IN_MILLIS = DAY_IN_MILLIS * 7;

  static const String FORMAT_DEFAULT = 'yyyy-MM-dd HH:mm:ss';
  static const String FORMAT_DATE = 'yyyy-MM-dd';
  static const String FORMAT_TIME = 'HH:mm:ss';

  ///获取当前时间的DateTime
  static DateTime get currentDateTime => DateTime.now();

  ///获取当前时间的int值
  static int get currentMilliSeconds => currentDateTime.millisecondsSinceEpoch;

  ///格式化当前时间
  static String formatNow([String format = FORMAT_DEFAULT]) => formatDateTime(currentDateTime, format);

  ///根据毫秒数获取DateTime
  static DateTime parseInt(int milliSeconds) => DateTime.fromMillisecondsSinceEpoch(milliSeconds);

  ///根据字符串获取DateTime
  static DateTime parseStr(String str) => DateTime.tryParse(str);

  ///获取日期当月有多少天
  static int dayInMonth(DateTime dt) => DateTime(dt.year, dt.month + 1, 1).difference(DateTime(dt.year, dt.month, 1)).inDays;

  /// 获取两个日期间的间隔天数
  static int interval(DateTime dt1, DateTime dt2) => dt1.difference(dt2).inDays;

  ///返回日期是星期几,默认返回数字,ch为true时返回中文
  static dynamic dayOfWeek(DateTime datetime, {bool ch = false}) {
    var chs = ['一', '二', '三', '四', '五', '六', '日'];
    return ch ? chs[datetime.weekday - 1] : datetime.weekday;
  }

  ///根据format格式化milliSeconds
  static String formatMicroseconds(int milliSeconds, [String format = FORMAT_DEFAULT]) => formatDateTime(parseInt(milliSeconds), format);

  ///根据format格式化DateTime,默认format为yyyy-MM-dd HH:mm:ss
  static String formatDateTime(DateTime datetime, [String format = FORMAT_DEFAULT]) {
    String formatStr = format;
    //year
    formatStr = _replace(formatStr, format, 'y', datetime.year);
    //month
    formatStr = _replace(formatStr, format, 'M', datetime.month);
    //day
    formatStr = _replace(formatStr, format, 'd', datetime.day);
    //hour
    formatStr = _replace(formatStr, format, 'H', datetime.hour);
    //minute
    formatStr = _replace(formatStr, format, 'm', datetime.minute);
    //second
    formatStr = _replace(formatStr, format, 's', datetime.second);
    //milliSeconds
    formatStr = _replace(formatStr, format, 'S', datetime.millisecond);

    return formatStr;
  }

  static String _replace(String formatStr, String format, String char, int date) {
    int num = 0;
    format.split('').forEach((str) {
      if (str == char) num++;
    });
    var replace;
    if (num > 0) {
      switch (char) {
        case 'y':
          replace = num == 2 ? '$date'.substring(2, 4) : '$date';
          break;
        case 'S':
          replace = '$date';
          break;
        default:
          replace = num == 1 ? '$date' : '$date'.padLeft(2, '0');
          break;
      }
    }
    if (replace != null) formatStr = formatStr.replaceAll(char * num, replace);
    return formatStr;
  }
}
