import 'dart:ui' as ui show window;

import 'package:flutter/widgets.dart';

num _uiWidth = 360;
num _uiHeight = 640;

/// 设置设计稿信息, 单位dp
///
/// @param width 设计稿宽度
///
/// @param height 设计稿高度
void setDesignInfo({num width, num height}) {
  assert(width != null, 'the design with is null!');
  assert(height != null, 'the design height is null!');

  _uiWidth = width;
  _uiHeight = height;
}

/// 屏幕适配工具类
class ScreenUtil {
  static ScreenUtil _singleton;

  factory ScreenUtil() {
    return _singleton ??= ScreenUtil._();
  }

  ScreenUtil._() {
    final window = MediaQueryData.fromWindow(ui.window);
    assert(window != null, 'You need to explicitly call the `WidgetsFlutterBinding.ensureInitialized()`, before initializing ScreenUtil.');

    _width = window.size.width;
    _scaleWidth = _width / _uiWidth;

    _height = window.size.height;
    _scaleHeight = _height / _uiHeight;

    _devicePixelRatio = window.devicePixelRatio;
    _textScaleFactor = window.textScaleFactor;

    _statusBarHeight = window.padding.top;
    _bottomBarHeight = window.padding.bottom;
  }

  double _width;
  double _scaleWidth;

  double _height;
  double _scaleHeight;

  double _devicePixelRatio;
  double _textScaleFactor;

  double _statusBarHeight;
  double _bottomBarHeight;

  /// 屏幕像素密度
  double get devicePixelRatio => _devicePixelRatio;

  /// 屏幕宽度
  double get width => _width;

  /// 屏幕高度
  double get height => _height;

  /// 字体缩放比例
  double get textScaleFactor => _textScaleFactor;

  /// 状态栏高度
  double get statusBarHeight => _statusBarHeight;

  /// 底部安全高度
  double get bottomBarHeight => _bottomBarHeight;

  /// 宽度方向适配
  num setWidth(num width) => width * _scaleWidth;

  /// 高度方向适配
  num setHeight(num height) => height * _scaleHeight;

  /// 字体适配
  ///
  /// @param allowScale 字体是否跟随系统缩放, 默认true
  num setSp(num fontSize, {bool allowScale = true}) => allowScale ? fontSize * _scaleWidth : fontSize * _scaleWidth / _textScaleFactor;
}

/// num的扩展函数,
extension ScreenExt on num {
  num get w => ScreenUtil().setWidth(this);

  num get h => ScreenUtil().setHeight(this);

  num get sp => ScreenUtil().setSp(this);

  num get nsp => ScreenUtil().setSp(this, allowScale: false);

  num get wp => ScreenUtil().width * this;

  num get hp => ScreenUtil().height * this;
}
